/* packet-applemidi.h
 *
 * Routines for dissection of Apple network-midi session establishment.
 * Copyright 2006-2010, Tobias Erichsen <t.erichsen@gmx.de>
 *
 * $Id$
 *
 * Wireshark - Network traffic analyzer
 * By Gerald Combs <gerald@wireshark.org>
 * Copyright 1998 Gerald Combs
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * Apple network-midi session establishment is a lightweight protocol for providing
 * a simple session establishment for MIDI-data sent in the form of RTP-MIDI (RFC 4695)
 * Peers recognize each other using the Apple Bonjour scheme with the service-name
 * "_apple-midi._udp", establish a connection using AppleMIDI (no official name, just
 * an abbreviation) and then send payload using RTP-MIDI.  The implementation of
 * this dissector is based on the Apple implementation summary from May 6th, 2005.
 *
 * Here are some links:
 *
 * http://www.cs.berkeley.edu/~lazzaro/rtpmidi/
 * http://www.faqs.org/rfcs/rfc4695.html
 */


/* Definitions for protocol name during dissector-register */
#define APPLEMIDI_DISSECTOR_NAME			"Apple Network-MIDI Session Protocol"
#define APPLEMIDI_DISSECTOR_SHORTNAME			"AppleMIDI"
#define APPLEMIDI_DISSECTOR_ABBREVIATION		"applemidi"

/* Signature "Magic Value" for Apple network MIDI session establishment */
#define APPLEMIDI_PROTOCOL_SIGNATURE			0xffff

/* Apple network MIDI valid commands */
#define APPLEMIDI_COMMAND_INVITATION			0x494e		/*   "IN"   */
#define APPLEMIDI_COMMAND_INVITATION_REJECTED		0x4e4f		/*   "NO"   */
#define APLLEMIDI_COMMAND_INVITATION_ACCEPTED		0x4f4b		/*   "OK"   */
#define APPLEMIDI_COMMAND_ENDSESSION			0x4259		/*   "BY"   */
#define APPLEMIDI_COMMAND_SYNCHRONIZATION		0x434b		/*   "CK"   */
#define APPLEMIDI_COMMAND_RECEIVER_FEEDBACK		0x5253		/*   "RS"   */



static const value_string applemidi_commands[] = {
	{ APPLEMIDI_COMMAND_INVITATION,			"Invitation" },
	{ APPLEMIDI_COMMAND_INVITATION_REJECTED,	"Invitation Rejected" },
	{ APLLEMIDI_COMMAND_INVITATION_ACCEPTED,	"Invitation Accepted" },
	{ APPLEMIDI_COMMAND_ENDSESSION,			"End Session" },
	{ APPLEMIDI_COMMAND_SYNCHRONIZATION,		"Synchronization" },
	{ APPLEMIDI_COMMAND_RECEIVER_FEEDBACK,		"Receiver Feedback" },
	{ 0,						NULL },
};


static int	hf_applemidi_signature			= -1;
static int	hf_applemidi_command			= -1;
static int	hf_applemidi_protocol_version		= -1;
static int	hf_applemidi_token			= -1;
static int	hf_applemidi_ssrc			= -1;
static int	hf_applemidi_name			= -1;
static int	hf_applemidi_count			= -1;
static int	hf_applemidi_padding			= -1;
static int	hf_applemidi_timestamp1			= -1;
static int	hf_applemidi_timestamp2			= -1;
static int	hf_applemidi_timestamp3			= -1;
static int	hf_applemidi_sequence_num		= -1;


static gint	ett_applemidi				= -1;


static hf_register_info hf[] =	{
	{
		&hf_applemidi_signature,
		{
			"Signature",
			"applemidi.signature",
			FT_UINT16,
			BASE_HEX,
			NULL,
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_command,
		{
			"Command",
			"applemidi.command",
			FT_UINT16,
			BASE_HEX,
			VALS( applemidi_commands ),
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_protocol_version,
		{
			"Protocol Version",
			"applemidi.protocol_version",
			FT_UINT32,
			BASE_DEC,
			NULL,
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_token,
		{
			"Initiator Token",
			"applemidi.initiator_token",
			FT_UINT32,
			BASE_HEX,
			NULL,
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_ssrc,
		{
			"Sender SSRC",
			"applemidi.sender_ssrc",
			FT_UINT32,
			BASE_HEX,
			NULL,
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_name,
		{
			"Name",
			"applemidi.name",
			FT_STRING,
			BASE_NONE,
			NULL,
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_count,
		{
			"Count",
			"applemidi.count",
			FT_UINT8,
			BASE_DEC,
			NULL,
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_padding,
		{
			"Padding",
			"applemidi.padding",
			FT_UINT24,
			BASE_HEX,
			NULL,
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_timestamp1,
		{
			"Timestamp 1",
			"applemidi.timestamp1",
			FT_UINT64,
			BASE_HEX,
			NULL,
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_timestamp2,
		{
			"Timestamp 2",
			"applemidi.timestamp2",
			FT_UINT64,
			BASE_HEX,
			NULL,
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_timestamp3,
		{
			"Timestamp 3",
			"applemidi.timestamp3",
			FT_UINT64,
			BASE_HEX,
			NULL,
			0x0,
			"", HFILL
		}
	},
	{
		&hf_applemidi_sequence_num,
		{
			"Sequence Number",
			"applemidi.sequence_number",
			FT_UINT32,
			BASE_HEX,
			NULL,
			0x0,
			"", HFILL
		}
	},
};


static gint *ett[] = {
	&ett_applemidi,
};


